config_default = [
    {
        "Configuration": {
            "Apply_Config": False,
            "Files": {
                "db_file": "download_history.sqlite",
                "errored_urls_file": "Errored_URLs.csv",
                "input_file": "URLs.txt",
                "log_file": "downloader.log",
                "output_folder": "Downloads",
                "output_last_forum_post_file": "URLs_last_post.txt",
                "unsupported_urls_file": "Unsupported_URLs.csv",
            },
            "Authentication": {
                "pixeldrain_api_key": "",
                "simpcity_username": "",
                "simpcity_password": "",
                "socialmediagirls_username": "",
                "socialmediagirls_password": "",
                "xbunker_username": "",
                "xbunker_password": "",
            },
            "JDownloader": {
                "apply_jdownloader": False,
                "jdownloader_username": "",
                "jdownloader_password": "",
                "jdownloader_device": "",
            },
            "Forum_Options": {
                "output_last_forum_post": False,
                "separate_posts": False,
            },
            "Ignore": {
                "exclude_videos": False,
                "exclude_images": False,
                "exclude_audio": False,
                "exclude_other": False,
                "ignore_cache": False,
                "ignore_history": False,
                "skip_hosts": [],
            },
            "Progress_Options": {
                "hide_new_progress": False,
                "dont_show_overall_progress": False,
                "dont_show_forum_progress": False,
                "dont_show_thread_progress": False,
                "dont_show_domain_progress": False,
                "dont_show_album_progress": False,
                "dont_show_file_progress": False
            },
            "Ratelimiting": {
                "connection_timeout": 15,
                "ratelimit": 50,
                "throttle": 0.5,
            },
            "Runtime": {
                "allow_insecure_connections": False,
                "attempts": 10,
                "block_sub_folders": False,
                "disable_attempt_limit": False,
                "include_id": False,
                "skip_download_mark_completed": False,
                "output_errored_urls": False,
                "output_unsupported_urls": False,
                "proxy": "",
                "remove_bunkr_identifier": False,
                "required_free_space": 5,
                "simultaneous_downloads_per_domain": 4,
            },
            "Sorting": {
                "sort_downloads": False,
                "sort_directory": "Sorted Downloads",
                "sorted_audio": "{sort_dir}/{base_dir}/Audio",
                "sorted_images": "{sort_dir}/{base_dir}/Images",
                "sorted_others": "{sort_dir}/{base_dir}/Other",
                "sorted_videos": "{sort_dir}/{base_dir}/Videos"
            }
        }
    }
]

authentication_args = ["pixeldrain_api_key",
                       "simpcity_username", "simpcity_password",
                       "socialmediagirls_username", "socialmediagirls_password",
                       "xbunker_username", "xbunker_password"]

files_args = ["db_file", "input_file", "log_file", "output_folder", "output_last_forum_post_file",
              "unsupported_urls_file", "errored_urls_file"]

forum_args = ["output_last_forum_post", "separate_posts"]

ignore_args = ["exclude_videos", "exclude_images", "exclude_audio", "exclude_other", "ignore_cache", "ignore_history",
               "skip_hosts"]

jdownloader_args = ["apply_jdownloader", "jdownloader_username", "jdownloader_password", "jdownloader_device"]

progress_args = ["hide_new_progress", "dont_show_overall_progress", "dont_show_forum_progress",
                 "dont_show_thread_progress", "dont_show_domain_progress", "dont_show_album_progress",
                 "dont_show_file_progress"]

ratelimiting_args = ["connection_timeout", "ratelimit", "throttle"]

runtime_args = ["simultaneous_downloads_per_domain", "allow_insecure_connections", "attempts", "disable_attempt_limit",
                "include_id", "skip_download_mark_completed", "proxy", "required_free_space", "output_errored_urls",
                "output_unsupported_urls", "block_sub_folders"]

sorting_args = ["sort_downloads", "sort_directory", "sorted_audio", "sorted_images", "sorted_others", "sorted_videos"]



from yarl import URL

from ..base_functions.base_functions import log, logger, get_filename_and_ext, get_db_path, make_title_safe
from ..base_functions.data_classes import AlbumItem, MediaItem
from ..base_functions.sql_helper import SQLHelper
from ..client.client import ScrapeSession


class PimpAndHostCrawler:
    def __init__(self, quiet: bool, SQL_Helper: SQLHelper):
        self.quiet = quiet
        self.SQL_Helper = SQL_Helper

    async def fetch(self, session: ScrapeSession, url: URL):
        """Director for pimpandhost scraping"""
        await log(f"[green]Starting: {str(url)}[/green]", quiet=self.quiet)
        album_obj = AlbumItem("Loose Pixeldrain Files", [])

        if url.parts[1] == 'album':
            media_items, title = await self.get_listings(session, url)
            await album_obj.set_new_title(title)
            if media_items:
                for media_item in media_items:
                    await album_obj.add_media(media_item)
        else:
            media_item = await self.get_singular(session, url)
            await album_obj.add_media(media_item)

        await self.SQL_Helper.insert_album("pimpandhost", url.path, album_obj)
        await log(f"[green]Finished: {str(url)}[/green]", quiet=self.quiet)
        return album_obj

    async def get_listings(self, session: ScrapeSession, url: URL):
        """Handles album media"""
        media_items = []
        try:
            soup = await session.get_BS4(url)
            title = soup.select_one("div[class=image-header]")
            user_link = title.select_one("span[class=details]")
            user_link.decompose()
            title = await make_title_safe(title.get_text())

            for file in soup.select('a[class*="image-wrapper center-cropped im-wr"]'):
                link = file.get("href")
                media_items.append(await self.get_singular(session, link))
            return media_items, title

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log(f"[red]Error: {str(url)}[/red]", quiet=self.quiet)
            logger.debug(e)

    async def get_singular(self, session: ScrapeSession, url: URL):
        """Handles singular files"""
        try:
            soup = await session.get_BS4(url)
            img = soup.select_one("a img")
            img = img.get("src")
            if img.startswith("//"):
                img = URL("https:" + img)
            filename, ext = await get_filename_and_ext(img.name)
            url_path = await get_db_path(img)
            complete = await self.SQL_Helper.check_complete_singular("pimpandhost", url_path)
            media_item = MediaItem(img, url, complete, filename, ext)
            return media_item
        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log(f"[red]Error: {str(url)}[/red]", quiet=self.quiet)
            logger.debug(e)



import re

from yarl import URL

from ..base_functions.base_functions import log, logger, make_title_safe, FILE_FORMATS, get_filename_and_ext, \
    get_db_path
from ..base_functions.data_classes import AlbumItem, MediaItem
from ..base_functions.error_classes import NoExtensionFailure
from ..base_functions.sql_helper import SQLHelper
from ..client.client import ScrapeSession


class BunkrCrawler:
    def __init__(self, quiet: bool, SQL_Helper: SQLHelper, remove_bunkr_id: bool):
        self.quiet = quiet
        self.SQL_Helper = SQL_Helper
        self.remove_bunkr_id = remove_bunkr_id

    async def fetch(self, session: ScrapeSession, url: URL):
        """Scraper for Bunkr"""
        album_obj = AlbumItem("Loose Bunkr Files", [])
        await log(f"[green]Starting: {str(url)}[/green]", quiet=self.quiet)

        if "v" in url.parts or "d" in url.parts:
            media = await self.get_file(session, url)
            if not media.filename:
                return album_obj
            await album_obj.add_media(media)
            await log(f"[green]Finished: {str(url)}[/green]", quiet=self.quiet)
            if not media.complete:
                await self.SQL_Helper.insert_media("bunkr", media.url.path, "", str(url), "", "", media.filename, 0)
            return album_obj

        if "a" in url.parts:
            album_obj = await self.get_album(session, url)
            await self.SQL_Helper.insert_album("bunkr", url.path, album_obj)

            if album_obj.media:
                await log(f"[green]Finished: {str(url)}[/green]", quiet=self.quiet)
            return album_obj

        cdn_possibilities = r"(?:cdn.bunkr...|cdn..bunkr...|cdn...bunkr...|media-files.bunkr...|media-files..bunkr...|media-files...bunkr...)"
        ext = '.' + url.parts[-1].split('.')[-1]
        if ext:
            ext = ext.lower()
        if ext in FILE_FORMATS['Images']:
            check_complete = await self.SQL_Helper.check_complete_singular("bunkr", url.path)
            url = URL(str(url).replace("https://cdn", "https://i"))

            filename, ext = await get_filename_and_ext(url.name)
            if self.remove_bunkr_id:
                filename = await self.remove_id(filename, ext)

            media_item = MediaItem(url, url, check_complete, filename, ext)
            await album_obj.add_media(media_item)
        else:
            if ext in FILE_FORMATS['Videos']:
                referer = URL(re.sub(cdn_possibilities, "bunkr.su/v", str(url)))
            else:
                referer = URL(re.sub(cdn_possibilities, "bunkr.su/d", str(url)))
            media_item = await self.get_file(session, referer)
            await album_obj.add_media(media_item)

        await self.SQL_Helper.insert_album("bunkr", url.path, album_obj)
        await log(f"[green]Finished: {str(url)}[/green]", quiet=self.quiet)
        return album_obj

    async def remove_id(self, filename: str, ext: str):
        """Removes the additional string bunkr adds to the end of every filename"""
        filename = filename.rsplit(ext, 1)[0]
        filename = filename.rsplit("-", 1)[0]
        if ext not in filename:
            filename = filename + ext
        return filename

    async def check_for_la(self, url: URL):
        if "12" in url.host:
            url_host = url.host.replace(".su", ".la").replace(".ru", ".la")
            url = url.with_host(url_host)
        return url

    async def get_file(self, session: ScrapeSession, url: URL):
        """Gets the media item from the supplied url"""
        try:
            soup = await session.get_BS4(url)
            head = soup.select_one("head")
            scripts = head.select('script[type="text/javascript"]')
            link = None

            for script in scripts:
                if script.text:
                    if "link.href" in script.text:
                        link = script.text.split('link.href = "')[-1].split('";')[0]
                        break
            if not link:
                raise
            link = URL(link)
            link = link.with_name(url.name)
            filename, ext = await get_filename_and_ext(link.name)
            if ext not in FILE_FORMATS['Images']:
                link = await self.check_for_la(link)
            if self.remove_bunkr_id:
                filename = await self.remove_id(filename, ext)

            complete = await self.SQL_Helper.check_complete_singular("bunkr", link.path)
            if complete:
                media = MediaItem(link, url, True, filename, ext)
                return media
            media = MediaItem(link, url, False, filename, ext)
            return media

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log(f"[red]Error: {str(url)}[/red]", quiet=self.quiet)
            logger.debug(e)
            return MediaItem(url, url, False, "", "")

    async def get_album(self, session: ScrapeSession, url: URL):
        """Iterates through an album and creates the media items"""
        album = AlbumItem(url.name, [])
        try:
            soup = await session.get_BS4(url)
            title = soup.select_one('h1[class="text-[24px] font-bold text-dark dark:text-white"]')
            for elem in title.find_all("span"):
                elem.decompose()
            title = await make_title_safe(title.get_text())
            await album.set_new_title(title)
            for file in soup.select('a[class*="grid-images_box-link"]'):
                link = file.get("href")
                media_loc = file.select_one("img").get("src").split("//i")[-1].split(".bunkr.")[0]

                if link.startswith("/"):
                    link = URL("https://" + url.host + link)
                link = URL(link)

                try:
                    filename, ext = await get_filename_and_ext(link.name)
                except NoExtensionFailure:
                    logger.debug("Couldn't get extension for %s", str(link))
                    continue

                referer = link
                if "v" in link.parts or "d" in link.parts:
                    media = await self.get_file(session, link)
                    link = media.url
                elif ext in FILE_FORMATS["Images"]:
                    link = URL(str(link).replace("https://cdn", "https://i"))
                else:
                    link = URL(f"https://media-files{media_loc}.bunkr.ru" + link.path)

                if ext not in FILE_FORMATS['Images']:
                    link = await self.check_for_la(link)

                if self.remove_bunkr_id:
                    filename = await self.remove_id(filename, ext)

                url_path = await get_db_path(link)
                complete = await self.SQL_Helper.check_complete_singular("bunkr", url_path)
                media = MediaItem(link, referer, complete, filename, ext)
                await album.add_media(media)

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log(f"[red]Error: {str(url)}[/red]", quiet=self.quiet)
            logger.debug(e)

        return album
